@@ -1,5 +1,5 @@
 /*
- * $Id: cdinfo.c,v 1.7 1997/12/19 04:42:09 dirk Exp $
+ * $Id: cdinfo.c,v 1.4 1999/02/04 16:52:16 dirk Exp $
  * 
  * This file is part of WorkMan, the civilized CD player program
  * (c) 1991-1997 by Steven Grimm (original author)
@@ -20,7 +20,7 @@
  * Get information about a CD.
  */
 
-static char cdinfo_id[] = "$Id: cdinfo.c,v 1.7 1997/12/19 04:42:09 dirk Exp $";
+static char cdinfo_id[] = "$Id: cdinfo.c,v 1.4 1999/02/04 16:52:16 dirk Exp $";
 
 #include <errno.h>
 #include <stdio.h>
@@ -72,17 +72,17 @@ insert_trackinfo(num)
 	struct trackinfo *newtrk;
 
 	/* Easy case: the list is empty */
-	if (cd->trk == NULL)
+	if (cd->trk == NULL) {
 		if ((cd->trk = (struct trackinfo *) calloc(1,
 						sizeof(*newtrk))) == NULL)
 		{
 nomem:
 			perror("insert_trackinfo");
 			exit(1);
-		}
-		else
+		} else {
 			return;
-
+                } /* if() else */
+        } /* if() */
 	/* Stick the new entry in cd->trk[]. */
 	if ((newtrk = (struct trackinfo *) malloc(sizeof(*newtrk) *
 						(cur_ntracks + 1))) == NULL)
@@ -284,34 +284,43 @@ listentry( int num )
 		name = cd->trk[num].songname ? cd->trk[num].songname : "";
 
 		if (cur_nsections)
-			if (cd->trk[num].section > 9)
+	        {
+    			if (cd->trk[num].section > 9) 
+			{
 				sprintf(tracknum, "%*d.%d", digits,
 					cd->trk[num].track,
 					cd->trk[num].section);
-			else if (cd->trk[num].section)
-				sprintf(tracknum, "%*d.%*d", digits,
-					cd->trk[num].track, sdigits,
-					cd->trk[num].section);
-			else
-				sprintf(tracknum, "%*d%*s", digits,
-					cd->trk[num].track,
-					2 - sdigits - big_spaces, " ");
-		else
+			} else {
+				if (cd->trk[num].section)
+				{
+					sprintf(tracknum, "%*d.%*d", digits,
+						cd->trk[num].track, sdigits,
+						cd->trk[num].section);
+				} else {
+					sprintf(tracknum, "%*d%*s", digits,
+						cd->trk[num].track,
+						2 - sdigits - big_spaces, " ");
+				}
+			}
+		} else {
 			sprintf(tracknum, "%*d", digits, cd->trk[num].track);
+		}
 
 		if (cd->trk[num].data)
+		{
 			sprintf(buf, "%s) %3dMB %s", tracknum,
 				cd->trk[num].length / 1024, name);
-		else
+		} else {
 			sprintf(buf, "%s) %02d:%02d %s", tracknum,
 				cd->trk[num].length / 60,
 				cd->trk[num].length % 60, name);
+                }
 
 		return (buf);
-	}
-	else
+	} else {
 		return (NULL);
-}
+        }
+} /* listentry() */
 
 /*
  * trackname()
@@ -321,14 +330,18 @@ listentry( int num )
 char *
 trackname( int num )
 {
-	if (num >= 0 && num < cur_ntracks)
+	if (num >= 0 && num < cur_ntracks) 
+	{
 		if (cd->trk[num].songname)
+		{
 			return (cd->trk[num].songname);
-		else
+		} else {
 			return ("");
-	else
+		}
+	} else {
 		return (NULL);
-}
+	}
+} /* trackname() */
 
 /*
  * tracklen()
@@ -447,7 +460,7 @@ default_volume( int track, int vol )
  * Play the next thing on the playlist, if any.
  */
 void
-play_next_entry( void )
+play_next_entry( int forward )
 {
 	if (cd == NULL)
 		return;
@@ -465,14 +478,14 @@ play_next_entry( void )
  * Play the next track, following playlists as necessary.
  */
 void
-play_next_track( void )
+play_next_track( int forward )
 {
 	if (cd == NULL)
 		return;
 
 	/* Is the current playlist entry done?  Move on, if so. */
 	if (playlist == NULL || cur_track + 1 == playlist[cur_listno - 1].end)
-		play_next_entry();
+		play_next_entry( forward );
 	else
 		play_cd(cur_track + 1, 0, playlist[cur_listno - 1].end);
 }
@@ -481,7 +494,7 @@ play_next_track( void )
  * Play the previous track, hopping around the playlist as necessary.
  */
 void
-play_prev_track( void )
+play_prev_track( int forward )
 {
 	if (cd == NULL)
 		return;
