@@ -1,5 +1,5 @@
 /*
- * $Id: cdrom.c,v 1.13 1997/12/19 04:42:35 dirk Exp $
+ * $Id: cdrom.c,v 1.7 1999/09/03 14:50:43 dirk Exp $
  *
  * This file is part of WorkMan, the civilized CD player program
  * (c) 1991-1997 by Steven Grimm (original author)
@@ -22,7 +22,7 @@
  * platform- or drive-dependent code here.
  */
 
-static char cdrom_id[] = "$Id: cdrom.c,v 1.13 1997/12/19 04:42:35 dirk Exp $";
+static char cdrom_id[] = "$Id: cdrom.c,v 1.7 1999/09/03 14:50:43 dirk Exp $";
 
 #include <errno.h>
 #include <stdio.h>
@@ -35,15 +35,19 @@ static char cdrom_id[] = "$Id: cdrom.c,v 1.13 1997/12/19 04:42:35 dirk Exp $";
 #include "struct.h"
 #include "config.h"
 #include "proto.h"
+#ifdef CDDB_IN_UI
+#include "wm_cddb.h"
+#endif
 
 #ifdef CAN_CLOSE
 #include <fcntl.h>
-/* extern char *cd_device; this will be dfined as (char *) */
+/* extern char *cd_device; this will be defined as (char *) */
 #endif
 
 
-extern struct wm_drive generic_proto, toshiba_proto, sony_proto,
-	toshiba33_proto;
+extern struct wm_drive generic_proto, toshiba_proto, sony_proto;
+
+/*	toshiba33_proto; <=== Somehow, this got lost */
 
 /*
  * The supported drive types are listed here.  NULL means match anything.
@@ -60,6 +64,7 @@ struct drivelist {
 {	"TOSHIBA",	"XM-3401",	NULL,	&toshiba_proto		},
 {	"TOSHIBA",	"XM-3301",	NULL,	&toshiba_proto		},
 {	"SONY",		"CDU-8012",	NULL,	&sony_proto		},
+{	"SONY",		"CDU 561",	NULL,	&sony_proto		},
 {	NULL,		NULL,		NULL,	&generic_proto		}
 };
 
@@ -74,8 +79,9 @@ int	intermittent_dev = 0;
 
 /*
  * Do we want to keep the CD device open after quitting by default?
+ *
+   int	keep_open = 0;
  */
-int	keep_open = 0;
 
 #if defined DEFAULT_CD_DEVICE
 char	*cd_device = DEFAULT_CD_DEVICE;
@@ -98,8 +104,7 @@ struct wm_drive	drive = { -1, "", "", NULL, NULL };
  * on the vendor, model, and revision of the current drive.
  */
 struct wm_drive *
-find_drive_struct(vendor, model, rev)
-	char	*vendor, *model, *rev;
+find_drive_struct(char *vendor, char *model, char *rev)
 {
 	struct drivelist	*d;
 
@@ -211,7 +216,9 @@ read_toc()
 	}
 
 	thiscd.length = thiscd.trk[thiscd.ntracks].length;
-
+#ifdef CDDB_IN_UI
+	thiscd.cddbid = cddb_discid(drive);
+#endif
 	return (&thiscd);
 }
 
@@ -231,7 +238,7 @@ cd_status( void )
 {
 	static enum cd_modes	oldmode = UNKNOWN;
 	enum cd_modes		mode;
-	int			status, trackno;
+	int			status, trackno = cur_track;
 	int			ret = 1;
 
 	/* Open the drive.  This returns 1 if the device isn't ready. */
@@ -246,11 +253,15 @@ cd_status( void )
 	if (status > 0)
 		return (0);
 
-	/* If the user hit the stop button, don't pass PLAYING as oldmode. */
-	if (cur_cdmode == STOPPED)
-		oldmode = STOPPED;
+	/* If the user hit the stop button, don't pass PLAYING as oldmode.
+         * Likewise, if we've just started playing, don't remember that
+         * we were stopped before (or the state machine in get_drive_status
+         * can get confused.)
+         */
+	if( (cur_cdmode == STOPPED) || (cur_cdmode == PLAYING) )
+		oldmode = cur_cdmode;
 
-	if ((drive.get_drive_status)(&drive, oldmode, &mode, &cur_frame,
+	if( (drive.get_drive_status)(&drive, oldmode, &mode, &cur_frame,
 					&trackno, &cur_index) < 0)
 	{
 		perror("CD get drive status");
@@ -293,10 +304,14 @@ cd_status( void )
 		} while ( status != 0 );
 #endif
 		if ((cd = read_toc()) == NULL)
+                {
 			if (exit_on_eject)
+			{
 				exit(-1);
-			else
+			} else {
 				return (-1);
+			}
+		}
 
 		cur_nsections = 0;
 		cur_ntracks = cd->ntracks;
@@ -348,7 +363,7 @@ cd_status( void )
 				cur_pos_rel = -cur_pos_rel;
 		}
 
-		if (playlist != NULL && playlist[0].start)
+		if( (playlist != NULL) && playlist[0].start & (cur_listno > 0))
 		{
 			cur_pos_abs -= cd->trk[playlist[cur_listno-1].
 				start - 1].start / 75;
@@ -481,6 +496,8 @@ cd_volume( int vol, int bal, int max )
 void
 pause_cd( void )
 {
+	static int paused_pos;
+
 	if (cur_cdmode == EJECTED)	/* do nothing if there's no CD! */
 		return;
 
@@ -488,11 +505,15 @@ pause_cd( void )
 	case PLAYING:		/* playing */
 		cur_cdmode = PAUSED;
 		(drive.pause)(&drive);
+                paused_pos = cur_pos_rel;
 		break;
 
 	case PAUSED:		/* paused */
 		cur_cdmode = PLAYING;
-		(drive.resume)(&drive);
+/*		(drive.resume)(&drive); */
+		if ((drive.resume)(&drive))
+			play_cd(cur_track, paused_pos,
+				playlist[cur_listno-1].end);
 	default: /* */
 		break;	
 	}
@@ -524,7 +545,7 @@ stop_cd( void )
  * Play the CD from one position to another (both in frames.)
  */
 void
-play_chunk( int start, int end)
+play_chunk( int start, int end, int realstart )
 {
 	if (cur_cdmode == EJECTED || cd == NULL)
 		return;
@@ -533,7 +554,7 @@ play_chunk( int start, int end)
 	if (start >= end)
 		start = end-1;
 
-	(drive.play)(&drive, start, end);
+	(drive.play)(&drive, start, end, realstart);
 }
 
 /*
@@ -554,7 +575,14 @@ play_cd( int start, int pos, int end )
 	cur_lasttrack = end;
 
 	play_chunk(cd->trk[start].start + pos * 75, end >= cur_ntracks ?
-		cur_cdlen * 75 : cd->trk[end].start - 1);
+		cur_cdlen * 75 : cd->trk[end].start - 1,
+		cd->trk[start].start);
+
+	/* So we don't update the display with the old frame number */
+	cd_status();
+	cur_frame = cd->trk[start].start + pos * 75;
+	cur_track = cur_firsttrack;
+	cur_cdmode = PLAYING;
 }
 
 /*
@@ -582,25 +610,16 @@ eject_cd( void )
 {
 	int	status;
 
-	if (cur_cdmode == EJECTED)	/* Already ejected! */
-		{
-#ifdef CAN_CLOSE
-		/* some CD-ROM drives will close the tray if the device
-		   is closed and then re-opened. This works with my
-		   CR-562, but YMMV...
-		   Dave Holland <dave@zenda.demon.co.uk> */
-		close(drive.fd);
-		drive.fd=open(cd_device, O_RDONLY | O_NONBLOCK);
-#endif
-		return (0);
-		}
-
 	status = (drive.eject)(&drive);
 	if (status < 0)
+	{
 		if (status == -3)
+		{
 			return (2);
-		else
+		} else {
 			return (1);
+		}
+	}
 	
 	if (exit_on_eject)
 		exit(0);
@@ -613,8 +632,13 @@ eject_cd( void )
 	return (0);
 }
 
+int cd_closetray(void)
+{
+	return((drive.closetray)(&drive) ? 0 : cd_status()==2 ? 1 : 0);
+} /* cd_closetray() */
+
 /*
- * find_trkind(track, index)
+ * find_trkind(track, index, start)
  *
  * Start playing at a particular track and index, optionally using a particular
  * frame as a starting position.  Returns a frame number near the start of the
@@ -652,7 +676,7 @@ find_trkind( int track, int index, int start )
 	interval = (top - bottom) / 4;
 
 	do {
-		play_chunk(current, current + 75);
+		play_chunk(current, current + 75, current);
 
 		if (cd_status() != 1)
 			return (0);
@@ -676,7 +700,7 @@ find_trkind( int track, int index, int start )
 	} while (interval > 2);
 
 	return (ret);
-}
+} /* find_trkind() */
 
 /*
  * Simulate usleep() using select().
@@ -690,7 +714,7 @@ susleep( int usec )
 	tv.tv_sec = usec / 1000000;
 	tv.tv_usec = usec % 1000000;
 	return (select(0, NULL, NULL, NULL, &tv));
-}
+} /* susleep() */
 
 /*
  * Read the initial volume from the drive, if available.  Set cur_balance to
@@ -731,7 +755,7 @@ read_initial_volume( int max )
 
 		return (left);
 	}
-}
+} /* read_initial_volume() */
 
 /*
  * Prototype wm_drive structure, with generic functions.  The generic functions
@@ -740,8 +764,8 @@ read_initial_volume( int max )
  */
 struct wm_drive generic_proto = {
 	-1,			/* fd */
-	"\0       ",		/* vendor */
-	"\0               ",	/* model */
+	"Generic\0",		/* vendor */
+	"drive type\0     ",	/* model */
 	NULL,			/* aux */
 	NULL,			/* daux */
 
@@ -756,5 +780,6 @@ struct wm_drive generic_proto = {
 	gen_resume,
 	gen_stop,
 	gen_play,
-	gen_eject
+	gen_eject,
+	gen_closetray
 };
