@@ -1,7 +1,7 @@
 #ifndef CDROM_H
 #define CDROM_H
 /*
- * $Id: cdrom.h,v 1.1 1997/12/19 04:43:16 dirk Exp $
+ * $Id: cdrom.h,v 1.4 1999/02/12 13:55:19 dirk Exp $
  *
  * This file is part of WorkMan, the civilized CD player program
  * (c) 1991-1997 by Steven Grimm (original author)
@@ -25,8 +25,9 @@
  */
 
 int	cd_status( void );
-void	play_chunk( int start, int end );
+void	play_chunk( int start, int end, int realstart );
 int	eject_cd( void );
+int     cd_closetray( void );
 void	play_from_pos( int pos );
 int	read_initial_volume( int max );
 void	pause_cd( void );
