@@ -1,5 +1,5 @@
 /*
- * $Id: database.c,v 1.9 1997/11/19 15:54:50 dirk Exp $
+ * $Id: database.c,v 1.9 1999/10/27 05:48:06 dirk Exp $
  *
  * This file is part of WorkMan, the civilized CD player program
  * (c) 1991-1997 by Steven Grimm (original author)
@@ -22,7 +22,7 @@
  * obviously it won't contain track titles just after a CD is inserted.)
  */
 
-static char database_id[] = "$Id: database.c,v 1.9 1997/11/19 15:54:50 dirk Exp $";
+static char database_id[] = "$Id: database.c,v 1.9 1999/10/27 05:48:06 dirk Exp $";
 
 #define RCFILE "/.workmanrc"
 #define DBFILE "/.workmandb"
@@ -42,6 +42,9 @@ static char database_id[] = "$Id: database.c,v 1.9 1997/11/19 15:54:50 dirk Exp
 #include "struct.h"
 #include "config.h"
 #include "proto.h"
+#ifdef CDDB_IN_UI
+#include "wm_cddb.h"
+#endif
 
 #define SWALLOW_LINE(fp) { int c; while ((c = getc(fp)) != '\n' && c != EOF); }
 
@@ -133,45 +136,53 @@ split_workmandb( void )
 {
 	int	ndbs, i;
 	char	*home, *wmdb;
-
+	int	no_rc = 0, no_db = 0;
+	
 	if (rcfile == NULL)
 	{
-		if ((home = getenv("HOME")) == NULL)
-		{
-nohome:
-			fprintf(stderr, "$HOME is not set!  Despair!\n");
-			exit(1);
-		}
-
-		rcfile = malloc(strlen(home) + sizeof(RCFILE));
-		if (rcfile == NULL)
+		if ((home = getenv("HOME")) != NULL)
 		{
+			rcfile = malloc(strlen(home) + sizeof(RCFILE));
+			if (rcfile == NULL)
+			{
+			
 nomem:
-			perror("split_workmandb()");
-			exit(1);
+				perror("split_workmandb()");
+				exit(1);
+			}
+
+			strcpy(rcfile, home);
+			strcat(rcfile, RCFILE);
 		}
+		else
+			no_rc = 1;
 
-		strcpy(rcfile, home);
-		strcat(rcfile, RCFILE);
 	}
 
 	if ((wmdb = dbfiles) == NULL)
 	{
-		if ((home = getenv("HOME")) == NULL)
-			goto nohome;
+		if ((home = getenv("HOME")) != NULL)
+		{
+			wmdb = malloc(strlen(home) + sizeof(DBFILE));
+			if (wmdb == NULL)
+				goto nomem;
 
-		wmdb = malloc(strlen(home) + sizeof(DBFILE));
-		if (wmdb == NULL)
-			goto nomem;
+			databases = malloc(2 * sizeof (databases[0]));
+			if (databases == NULL)
+				goto nomem;
 
-		databases = malloc(2 * sizeof (databases[0]));
-		if (databases == NULL)
-			goto nomem;
-
-		strcpy(wmdb, home);
-		strcat(wmdb, DBFILE);
-		databases[0] = wmdb;
-		databases[1] = NULL;
+			strcpy(wmdb, home);
+			strcat(wmdb, DBFILE);
+			databases[0] = wmdb;
+			databases[1] = NULL;
+		}
+		else
+		{
+			static char *emptydb = NULL;
+		
+			databases = &emptydb;
+			no_db = 1;
+		}
 	}
 	else
 	{
@@ -195,6 +206,26 @@ nomem:
 
 		databases[i] = NULL;
 	}
+
+	if (no_db || no_rc)
+	{
+		fprintf(stderr,
+"WorkMan was run without a home directory, probably by a system daemon.\n");
+		fprintf(stderr, "It doesn't know where to find ");
+		if (no_rc)
+		{
+			fprintf(stderr, "your personal preferences file ");
+			if (no_db)
+				fprintf(stderr, "or the\ndatabase of CD descriptions");
+		}
+		else
+			fprintf(stderr, "the database of CD descriptions");
+
+		fprintf(stderr,
+".\nYou can use the X resources \"workman.db.shared\" and \"workman.db.personal\"\nto tell WorkMan where to look.\n");
+
+		disable_save();
+	}
 }
 
 /*
@@ -218,7 +249,7 @@ print_cdinfo(cd, prefs)
 	static char	*cdibuf = NULL;
 	struct playlist	*l;
 
-	sprintf(tempbuf, "tracks %d", cd->ntracks);
+	sprintf(tempbuf, "\ntracks %d", cd->ntracks);
 	for (i = 0; i < cur_ntracks; i++)
 		if (cd->trk[i].section < 2)
 			sprintf(tempbuf + strlen(tempbuf), " %d",
@@ -230,6 +261,7 @@ print_cdinfo(cd, prefs)
 	if (cur_nsections)
 	{
 		sprintf(tempbuf, "sections %d", cur_nsections);
+		/* fixed a bug here */
 		for (i = 0; i < cur_ntracks; i++)
 			if (cd->trk[i].section > 1)
 				sprintf(tempbuf + strlen(tempbuf), " %d",
@@ -272,7 +304,11 @@ print_cdinfo(cd, prefs)
 
 		if (cd->volume)
 		{
-			sprintf(tempbuf, "cdvolume %d\n", cd->volume);
+			/*
+			 * Have to maintain compatibility with old versions,
+			 * where volume was 0-32.
+			 */
+			sprintf(tempbuf, "cdvolume %d\n", (cd->volume * 32) / 100);
 			strmcat(&cdibuf, tempbuf);
 		}
 
@@ -306,7 +342,7 @@ print_cdinfo(cd, prefs)
 			if (cd->trk[i].volume)
 			{
 				sprintf(tempbuf, "volume %d %d\n", i + 1,
-					cd->trk[i].volume);
+					(cd->trk[i].volume * 32) / 100);
 				strmcat(&cdibuf, tempbuf);
 			}
 			if (cd->trk[i].otherrc)
@@ -409,8 +445,7 @@ open_rcfile(name, mode)
 	return (fp);
 }
 
-int *
-reset_tracks()
+int *reset_tracks(void)
 {
     int i, j;
     int *trackmap;
@@ -444,7 +479,8 @@ reset_tracks()
  * Returns 1 if there was a match or 0 if not.
  *
  *	fp		FILE* of database or rcfile.
- *	prefs		1 if we're searching .workmanrc, 0 for .workmandb
+ *	prefs		1 if we're searching .workmanrc, 0 for .workmandb,
+ *			2 if just reading settings
  *	scan		Scan for "tracks" location and entry size only
  *	holesize_wanted	How big a hole we're looking for, if any.
  *
@@ -462,10 +498,10 @@ search_db( FILE *fp, int prefs, int scan, int holesize_wanted )
 {
 	char	keyword[64], listname[64], *c;
 	int	b, i, j, track = 0, listsize, ntracks, scratch, searching = 1;
-	int	*trackmap, gotsections = 0;
+	int	*trackmap = 0, gotsections = 0;
         int     fudge, maxfudge, sizediff, bestfudge = 0;
 	long	pos = 0, thisholepos = -1, holesize = 99991239;
-	struct playlist	*l;
+	struct	playlist *l;
 
 	rclen = 0;
 
@@ -473,16 +509,15 @@ search_db( FILE *fp, int prefs, int scan, int holesize_wanted )
 	if (holesize_wanted)
 		holepos = -1;
 
-        trackmap = reset_tracks();
+	if( prefs != 2 )
+		trackmap = reset_tracks();
 
 	if (prefs)
 		freeup(&otherrc);
 	firstpos = -1;
-
 	while (! feof(fp))
 	{
 		pos = ftell(fp);
-
 		keyword[0] = '\0';
 		do
 			b = getc(fp);
@@ -511,6 +546,9 @@ search_db( FILE *fp, int prefs, int scan, int holesize_wanted )
 		/* If this is the start of a CD entry, see if it matches. */
 		if (! strcmp(keyword, "tracks"))
 		{
+			if (prefs == 2)
+				break;
+
 			/* Is this the end of a hole? */
 			if (holesize_wanted && (thisholepos >= 0))
 			{
@@ -601,6 +639,82 @@ chomp:
 		}
 
 		/* Global mode stuff goes here */
+#ifdef CDDB_IN_UI		
+		else if (! strcmp(keyword, "cddbprotocol"))
+		{
+			getc(fp);
+			i = getc(fp);	/* only first letter is used */
+		        cddb.protocol = i == 'c' ? 1 : 
+		                        i == 'h' ? 2 : 3 ;
+			do
+				i = getc(fp);
+			while (i != '\n' && i != EOF);
+		}
+
+		else if (! strcmp(keyword, "cddbserver"))
+		{
+			getc(fp);	/* lose the space */
+			if (cddb.cddb_server[0])
+				do
+					i = getc(fp);
+				while (i != '\n' && i != EOF);
+			else
+			{
+				fgets(cddb.cddb_server, 
+				      sizeof(cddb.cddb_server), fp);
+				if ((i = strlen(cddb.cddb_server)))
+					cddb.cddb_server[i - 1] = '\0';
+			}
+		}
+
+		else if (! strcmp(keyword, "cddbmailadress"))
+		{
+			getc(fp);	/* lose the space */
+			if (cddb.mail_adress[0])
+				do
+					i = getc(fp);
+				while (i != '\n' && i != EOF);
+			else
+			{
+				fgets(cddb.mail_adress, 
+				      sizeof(cddb.mail_adress), fp);
+				if ((i = strlen(cddb.mail_adress)))
+					cddb.mail_adress[i - 1] = '\0';
+			}
+		}
+
+		else if (! strcmp(keyword, "cddbpathtocgi"))
+		{
+			getc(fp);	/* lose the space */
+			if (cddb.path_to_cgi[0])
+				do
+					i = getc(fp);
+				while (i != '\n' && i != EOF);
+			else
+			{
+				fgets(cddb.path_to_cgi, 
+				      sizeof(cddb.path_to_cgi), fp);
+				if ((i = strlen(cddb.path_to_cgi)))
+					cddb.path_to_cgi[i - 1] = '\0';
+			}
+		}
+
+		else if (! strcmp(keyword, "cddbproxy"))
+		{
+			getc(fp);	/* lose the space */
+			if (cddb.proxy_server[0])
+				do
+					i = getc(fp);
+				while (i != '\n' && i != EOF);
+			else
+			{
+				fgets(cddb.proxy_server, 
+				      sizeof(cddb.proxy_server), fp);
+				if ((i = strlen(cddb.proxy_server)))
+					cddb.proxy_server[i - 1] = '\0';
+			}
+		}
+#endif
 		else if (! strcmp(keyword, "whendone"))
 		{
 			getc(fp);
@@ -718,7 +832,8 @@ chomp:
 
 			getc(fp);	/* lose the space */
 			/* don't overwrite existing track names. */
-			if (trackmap[track] == -1)
+			/* However, overwrite them if there was a "bad" fuzzy match before */
+			if ((trackmap[track] == -1 || track > (cd->ntracks + cur_nsections)) && (searching == 2))
 				SWALLOW_LINE(fp)
 			else if (cd->trk[trackmap[track]].songname &&
 					cd->trk[trackmap[track]].songname[0])
@@ -749,12 +864,16 @@ chomp:
                            we find the name, in hopes that we will recover
                            most, if not all, of the information from the
                            file. */
-                        wipe_cdinfo(); 
-                        trackmap = reset_tracks();
+/*
+ * nasty bug was here, was it? 
+ *
+ *			 wipe_cdinfo(); 
+ * 			 trackmap = reset_tracks();
+ */
 
 			getc(fp);	/* lose the space */
 			/* don't overwrite existing cd name. */
-			if (cd->cdname[0])
+			if (cd->cdname[0] && (searching == 2))
 				do
 					i = getc(fp);
 				while (i != '\n' && i != EOF);
@@ -791,7 +910,10 @@ chomp:
 		}
 
 		else if (! strcmp(keyword, "cdvolume"))
+		{
 			fscanf(fp, "%d", &cd->volume);
+			cd->volume = (cd->volume * 100) / 32;
+		}
 
 		else if (! strcmp(keyword, "dontplay"))
 		{
@@ -815,6 +937,7 @@ chomp:
 			{
 				i = trackmap[i - 1];
 				fscanf(fp, "%d", &cd->trk[i].volume);
+				cd->trk[i].volume = (cd->trk[i].volume*100)/32;
 				if (cd->trk[i].volume > 32)
 					cd->trk[i].volume = 0;
 			}
@@ -896,12 +1019,13 @@ plnomem:
 			i = track ? trackmap[track - 1] : 0;
 			buf = prefs ? i ? &cd->trk[i].otherrc : &cd->otherrc :
 				i ? &cd->trk[i].otherdb : &cd->otherdb;
-			if (firstpos == -1)
-				if (prefs)
+			if (firstpos == -1) {
+				if (prefs) {
 					buf = &otherrc;
-				else
+				} else {
 					goto chomp;
-
+				} /* if() else */
+                        } /* if() */
 			strmcat(buf, keyword);
 			do {
 				input[sizeof(input) - 1] = 'x';
@@ -917,6 +1041,9 @@ plnomem:
         if (searching > 1) /* A near match has been found. Good enough. */
             searching = 0;
 
+#ifdef CDDB_IN_UI
+        cddb_struct2cur();
+#endif
 	return (! searching);
 
 }
@@ -1007,11 +1134,11 @@ load( void )
 		trklist[i] = cd->trk[i].start;
 
 	do {
-		if (idx_find_entry(*dbfile, cd->ntracks, trklist,
+		if (*dbfile && idx_find_entry(*dbfile, cd->ntracks, trklist,
 				cd->length * 75, 0, &dbpos) == 0)
 			dbfound = 1;
 
-		fp = open_rcfile(*dbfile, "r");
+		fp = *dbfile ? open_rcfile(*dbfile, "r") : NULL;
 		if (fp != NULL)
 		{
 			if (lockit(fileno(fp), F_RDLCK))
@@ -1044,7 +1171,7 @@ load( void )
 	  }	
 #endif
 
-	fp = open_rcfile(rcfile, "r");
+	fp = rcfile ? open_rcfile(rcfile, "r") : NULL;
 	if (fp != NULL)
 	{
 		locked = 0;
@@ -1079,6 +1206,36 @@ load( void )
 #endif
 }
 
+/*
+ * Load program settings from the rcfile.
+ */
+void
+load_settings( void )
+{
+	FILE    *fp;
+	int     locked;
+
+	fp = rcfile ? open_rcfile(rcfile, "r") : NULL;
+	if (fp != NULL)
+	{
+		locked = 0;
+		if (lockit(fileno(fp), F_RDLCK))
+			perror("Couldn't get read (rc) lock");
+		else
+			locked = 1;
+
+		rcpos = 0;
+		found_in_rc = search_db(fp, 2, 0, 0);
+		if (! found_in_rc)
+			cd->autoplay = get_playnew();
+
+		if (locked && lockit(fileno(fp), F_UNLCK))
+			perror("Couldn't relinquish (rc) lock");
+
+		fclose(fp);
+	}
+}
+
 /*
  * save_globals()
  *
@@ -1097,6 +1254,54 @@ FILE *fp;
 	if (otherrc)
 		strmcpy(&globes, otherrc);
 
+#ifdef CDDB_IN_UI
+	if (cddb.protocol)
+	{
+		sprintf(temp, "cddbprotocol ");
+		switch(cddb.protocol)
+		{
+		 case 1: /* cddbp */
+		    sprintf(temp + strlen(temp), "cddbp\n");
+		    break;
+		 case 2: /* http */
+		    sprintf(temp + strlen(temp), "http\n");
+		    break;
+		 case 3: /* proxy */
+		    sprintf(temp + strlen(temp), "proxy\n");
+		    break;
+		 default:
+		}
+		strmcat(&globes, temp);
+	    
+		if(cddb.mail_adress[0])
+	 	{
+			sprintf(temp,"cddbmailadress %s\n",
+				cddb.mail_adress);
+			strmcat(&globes, temp);
+		}
+
+		if(cddb.cddb_server[0])
+	 	{
+			sprintf(temp,"cddbserver %s\n",
+				cddb.cddb_server);
+			strmcat(&globes, temp);
+		}
+
+		if(cddb.path_to_cgi[0])
+	 	{
+			sprintf(temp,"cddbpathtocgi %s\n",
+				cddb.mail_adress);
+			strmcat(&globes, temp);
+		}
+
+		if(cddb.proxy_server[0])
+	 	{
+			sprintf(temp,"cddbproxy %s\n",
+				cddb.mail_adress);
+			strmcat(&globes, temp);
+		}
+	}
+#endif /* CDDB_IN_UI */
 	if (cur_stopmode == 1 || cur_stopmode == 2)
 	{
 		sprintf(temp, "whendone %s\n", cur_stopmode == 1 ? "repeat" :
@@ -1217,6 +1422,10 @@ save_entry(filename, pref)
 	char		*buf;
 	int		len, i, locked = 0;
 
+
+	if( filename == NULL )
+		return (-1);
+
 	fp = open_rcfile(filename, "r+");
 	if (fp == NULL)
 	{
