@@ -1,5 +1,5 @@
 /*
- * $Id: display.c,v 1.6 1997/12/15 19:52:30 dirk Exp $
+ * $Id: display.c,v 1.6 1999/10/27 05:48:06 dirk Exp $
  *
  * This file is part of WorkMan, the civilized CD player program
  * (c) 1991-1997 by Steven Grimm (original author)
@@ -20,7 +20,7 @@
  * display.c - update the status indicators and other display elements.
  */
 
-static char display_id[] = "$Id: display.c,v 1.6 1997/12/15 19:52:30 dirk Exp $";
+static char display_id[] = "$Id: display.c,v 1.6 1999/10/27 05:48:06 dirk Exp $";
 
 #include <stdio.h>
 #include <sys/param.h>
@@ -75,6 +75,7 @@ icon_label( char *string )
 	xv_set(Workman_window1->window1, FRAME_ICON, icon, NULL);
 }
 
+
 /*
  * Center the track list.
  */
@@ -804,7 +805,7 @@ show_stats( window1_objects *ip )
 		if (cur_cdmode != PLAYING && cur_cdmode != PAUSED)
 			xv_set(Workman_goodies->b, PANEL_INACTIVE, TRUE, NULL);
 	}
-}
+} /* show_stats() */
 
 /*
  * Populate the numbered buttons.  This has to be done in a fairly stupid
@@ -960,8 +961,15 @@ init_stats( window1_objects *ip )
 	xv_set(pu->playmode, PANEL_INACTIVE, FALSE, NULL);
 	xv_set(pu->button1, PANEL_INACTIVE, FALSE, NULL);
 	xv_set(pu->button8, PANEL_INACTIVE, FALSE, NULL);
+#if CDDB_IN_UI
+	xv_set(pu->cddb, PANEL_INACTIVE, FALSE, NULL);
+#endif
 	fill_buttons(); /* fill_buttons(ip) */
 	fill_lists();
+#if CDDB_IN_UI
+	cddb_select(NULL, -1, NULL);
+	/*                 0         */
+#endif
 
 	xv_set(ip->tracks, PANEL_CHOOSE_ONE, TRUE, PANEL_CHOOSE_NONE, TRUE,
 		PANEL_VALUE, -1, NULL);
@@ -1092,6 +1100,9 @@ kill_stats( window1_objects *ip )
 	xv_set(pu->playmode, PANEL_INACTIVE, TRUE, NULL);
 	xv_set(pu->button1, PANEL_INACTIVE, TRUE, NULL);
 	xv_set(pu->button8, PANEL_INACTIVE, TRUE, NULL);
+#ifdef CDDB_IN_UI
+	xv_set(pu->cddb, PANEL_INACTIVE, TRUE, NULL);
+#endif
 	xv_set(ip->tracks, PANEL_CHOICE_STRINGS, "No CD in drive", NULL, NULL);
 	xv_set(ip->tracktimer, PANEL_LABEL_STRING, "00:00", NULL);
 	xv_set(ip->cdtimer, PANEL_LABEL_STRING, "00:00", NULL);
