@@ -1,10 +1,34 @@
+/*
+ * This file is part of WorkMan, the civilized CD player program
+ * (c) 1991-1997 by Steven Grimm (original author)
+ * (c) by Dirk Frsterling (current 'author' = maintainer)
+ * The maintainer can be contacted by his e-mail address:
+ * milliByte@DeathsDoor.com 
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ */
+
 #include <xview/xview.h>
 #include <xview/font.h>
 #include <xview/panel.h>
 #include <xview/svrimage.h>
+#include <xview/defaults.h>
 #include "workman_ui.h"
+#include "ui_goodies.h"
+#include "struct.h"
+#include "config.h"
+#include "proto.h"
+#include "wm_cddb.h"
 
-static char *ident = "@(#)setup.c	1.29 04 Jun 1995";
+static char setup_id[] = "$Id: setup.c,v 1.7 1999/10/27 05:48:06 dirk Exp $";
 
 /*
  * Define a few macros to make relative positioning easier.
@@ -21,24 +45,6 @@ static char *ident = "@(#)setup.c	1.29 04 Jun 1995";
 #define center_x(t, s) xv_set(t, XV_X, (lef(s) + rig(s) - wid(t)) / 2, NULL)
 #define center_y(t, s) xv_set(t, XV_Y, (top(s) + bot(s) - hei(t)) / 2, NULL)
 
-extern void change_track(),
-	show_cdinfo(),
-	show_db_prefs(),
-	window1_button3_notify_callback(),
-	window1_button4_notify_callback(),
-	popup1_buttonpl_notify_callback(),
-	popup1_button1_notify_callback(),
-	goodies_playnewcds_notify_callback(),
-	goodies_abrepeat_notify_callback(),
-	plpopup_button7_notify_callback(),
-	change_mode(), delete_from_playlist(), playlist_notify(),
-	delete_playlist(), rename_playlist(), switch_playlists(),
-	slide_balance(), section_start(), section_end(), change_pos(),
-	next_stopmode(), set_volume(), next_playmode_default(),
-	update_title(), update_trackname(), name_entered(),
-	set_which_volume(), set_default_volume(), save_config(),
-	cdinfo_reset(), split_track(), delete_track(), index_scan();
-
 int	basic_spacing = 0;
 int	small_buttons = -1;
 int	big_spaces = 0;		/* Flag: spaces are as big as digits */
@@ -52,13 +58,12 @@ int	big_spaces = 0;		/* Flag: spaces are as big as digits */
  * Create the main window and its contents.
  */
 window1_objects *
-window1_objects_init(ip, owner)
-	window1_objects	*ip;
-	Xv_opaque		owner;
+window1_objects_init( window1_objects *ip, Xv_opaque owner )
 {
 	Xv_opaque	image, image_mask, speaker_image, mode_image0,
 			mode_image1, mode_image2, mode_image3, mode_image4,
 			mode_image5;
+	Icon            workman_icon;
 	Display		*dpy;
 	Xv_Screen	screen;
 	Panel_item	item;
@@ -85,27 +90,58 @@ window1_objects_init(ip, owner)
 	image = xv_create(XV_NULL, SERVER_IMAGE, SERVER_IMAGE_DEPTH, 1,
 		SERVER_IMAGE_X_BITS, icon_bits, XV_WIDTH, 64, XV_HEIGHT, 64,
 		NULL);
+		
 	image_mask = xv_create(XV_NULL, SERVER_IMAGE, SERVER_IMAGE_DEPTH,
 		1, SERVER_IMAGE_X_BITS, iconmask_bits, XV_WIDTH, 64,
 		XV_HEIGHT, 64, NULL);
-	ip->window1 = xv_create(owner, FRAME, XV_KEY_DATA, INSTANCE,
-		ip, XV_LABEL, "WorkMan", FRAME_SHOW_FOOTER, FALSE,
-		FRAME_SHOW_RESIZE_CORNER, FALSE, FRAME_ICON,
-		xv_create(XV_NULL, ICON, ICON_IMAGE, image,
-		ICON_MASK_IMAGE, image_mask, XV_LABEL, "Wait...",
-		NULL), XV_WIDTH, Dft_Size, XV_HEIGHT, Dft_Size, NULL);
+
+/* added the icon code */
+
+	workman_icon = xv_create(XV_NULL, ICON,
+				ICON_IMAGE, image,
+				ICON_MASK_IMAGE, image_mask,
+				ICON_TRANSPARENT, TRUE,
+				XV_LABEL, "Wait...", NULL);
+
+	xv_set( workman_icon,
+		WIN_CONSUME_EVENTS,
+		LOC_WINENTER,
+		LOC_WINEXIT,
+		LOC_DRAG,
+		NULL,
+		NULL);
+
+	ip->window1 = xv_create(owner, FRAME,
+/*		XV_KEY_DATA, INSTANCE, ip,
+		XV_LABEL, "WorkMan",
+*/
+/* added no_confirm  and label stuff*/
+		FRAME_NO_CONFIRM, TRUE,
+		FRAME_LABEL, "WorkMan",
+		FRAME_SHOW_LABEL, TRUE,
+/*
+		FRAME_SHOW_FOOTER, FALSE,
+		FRAME_SHOW_RESIZE_CORNER, FALSE,
+*/
+		FRAME_ICON, workman_icon,
+		XV_WIDTH, Dft_Size,
+		XV_HEIGHT, Dft_Size, NULL);
+
 	dpy = (Display *) xv_get(ip->window1, XV_DISPLAY);
 	screen = (Xv_Screen) xv_get(ip->window1, XV_SCREEN);
 	screen_no = (int) xv_get(screen, SCREEN_NUMBER);
 	swidth = DisplayWidth(dpy, screen_no);
 	sheight = DisplayHeight(dpy, screen_no);
 
-	ip->controls1 = xv_create(ip->window1, PANEL, XV_KEY_DATA,
-		INSTANCE, ip, XV_HELP_DATA, "workman:controls1",
-		XV_WIDTH, 10, XV_HEIGHT, 10, WIN_BORDER,
-		FALSE, NULL);
+	ip->controls1 = xv_create(ip->window1, PANEL,
+		XV_KEY_DATA, INSTANCE, ip,
+		XV_HELP_DATA, "workman:controls1",
+		XV_WIDTH, 10,
+		XV_HEIGHT, 10,
+		WIN_BORDER, FALSE, NULL);
 
 	/* Create a dummy message item to get spacing information. */
+
 	item = xv_create(ip->controls1, PANEL_MESSAGE,
 		PANEL_LABEL_STRING, " ", PANEL_LABEL_BOLD, TRUE, NULL);
 	if (! basic_spacing)
@@ -174,7 +210,7 @@ window1_objects_init(ip, owner)
 		PANEL_TICKS, 0, PANEL_DIRECTION, PANEL_VERTICAL,
 		PANEL_SLIDER_END_BOXES, FALSE, PANEL_SHOW_RANGE, FALSE,
 		PANEL_SHOW_VALUE, FALSE, PANEL_MIN_VALUE, 0,
-		PANEL_MAX_VALUE, 32, PANEL_VALUE, 0,
+		PANEL_MAX_VALUE, 100, PANEL_VALUE, 0,
 		PANEL_NOTIFY_PROC, set_volume, NULL);
 	put_down(ip->volume, ip->speaker, 3);
 
@@ -255,7 +291,8 @@ window1_objects_init(ip, owner)
 			"Repeat",
 			"Eject",
 			NULL,
-		PANEL_DEFAULT_VALUE, 0,
+		PANEL_DEFAULT_VALUE, 1,
+		PANEL_VALUE, 0,
 		NULL);
 	put_right(ip->repeat, ip->mode, 8);
 
@@ -292,7 +329,8 @@ window1_objects_init(ip, owner)
 		PANEL_LABEL_STRING, "CD Info...",
 /*
 		PANEL_ITEM_MENU, ip->dbmenu,
-*/		PANEL_NOTIFY_PROC, show_cdinfo,
+*/		
+		PANEL_NOTIFY_PROC, show_cdinfo,
 		NULL);
 	
 	ip->button4 = xv_create(ip->controls1, PANEL_BUTTON,
@@ -357,9 +395,11 @@ window1_objects_init(ip, owner)
 	xv_set(ip->controls1, XV_HEIGHT, bot(ip->button2) + 5, NULL);
 	window_fit(ip->window1);
 
+/*
 	PANEL_EACH_ITEM(ip->controls1, item)
 		xv_set(item, XV_KEY_DATA, INSTANCE, ip, NULL);
 	PANEL_END_EACH;
+*/
 
 	/*
 	 * Now adjust the window's position in case it was started with
@@ -382,9 +422,7 @@ window1_objects_init(ip, owner)
  * Set up the CD Info popup.
  */
 popup1_objects *
-popup1_objects_init(ip, owner)
-	popup1_objects	*ip;
-	Xv_opaque		owner;
+popup1_objects_init( popup1_objects *ip, Xv_opaque owner)
 {
 	int	space, spwidth, zerowidth;
 	Xv_Font	font;
@@ -470,7 +508,7 @@ popup1_objects_init(ip, owner)
 	ip->trackname = xv_create(ip->controls2, PANEL_TEXT,
 		XV_HELP_DATA, "workman:trackname",
 		XV_X, 5,
-		PANEL_VALUE_DISPLAY_LENGTH, 35,
+		PANEL_VALUE_DISPLAY_LENGTH, 30,
 		PANEL_VALUE_STORED_LENGTH, 500,
 		PANEL_LABEL_STRING, "Name:",
 		PANEL_LAYOUT, PANEL_HORIZONTAL,
@@ -482,6 +520,23 @@ popup1_objects_init(ip, owner)
 
 	xv_set(ip->tracklist, PANEL_LIST_WIDTH, wid(ip->trackname) + 10, NULL);
 
+	ip->insbutton = xv_create(ip->controls2, PANEL_BUTTON,
+		XV_HELP_DATA, "workman:insbutton",
+		PANEL_LABEL_STRING, "insert",
+		PANEL_NOTIFY_PROC, insert_in_tracklist,
+		NULL);
+
+	put_down(ip->insbutton, ip->trackname, basic_spacing);
+	center_x(ip->insbutton, ip->trackname);
+	
+	ip->delbutton = xv_create(ip->controls2, PANEL_BUTTON,
+		XV_HELP_DATA, "workman:delbutton",
+		PANEL_LABEL_STRING, "delete",
+		PANEL_NOTIFY_PROC, delete_from_tracklist,
+		NULL);
+
+	put_right(ip->delbutton, ip->insbutton, basic_spacing);	
+
 	ip->trackoptions = xv_create(ip->controls2, PANEL_TOGGLE,
 		XV_HELP_DATA, "workman:trackoptions",
 		PANEL_CHOICE_NCOLS, 1,
@@ -493,7 +548,7 @@ popup1_objects_init(ip, owner)
 		PANEL_VALUE, 0,
 		PANEL_INACTIVE, TRUE,
 		NULL);
-	put_down(ip->trackoptions, ip->trackname, 5);
+	put_down(ip->trackoptions, ip->insbutton, 5);
 
 	ip->whichvolume = xv_create(ip->controls2, PANEL_CHOICE,
 		PANEL_DISPLAY_LEVEL, PANEL_CURRENT,
@@ -519,9 +574,10 @@ popup1_objects_init(ip, owner)
 		PANEL_SHOW_RANGE, FALSE,
 		PANEL_SHOW_VALUE, FALSE,
 		PANEL_MIN_VALUE, 0,
-		PANEL_MAX_VALUE, 32,
+		PANEL_MAX_VALUE, 100,
 		PANEL_VALUE, 0,
 		PANEL_NOTIFY_PROC, set_default_volume,
+		PANEL_NOTIFY_LEVEL, PANEL_ALL,
 		NULL);
 
 	/* This will get changed to an image later, but use text for now */
@@ -596,21 +652,30 @@ popup1_objects_init(ip, owner)
 
 	ip->button1 = xv_create(ip->controls2, PANEL_BUTTON,
 		XV_HELP_DATA, "workman:button1",
-		PANEL_LABEL_STRING, "Apply",
+		PANEL_LABEL_STRING, "Save",
 		PANEL_NOTIFY_PROC, save_config,
 		NULL);
 	put_down(ip->button1, ip->autoplay, (basic_spacing * 3) / 2);
-	xv_set(ip->button1, XV_X, wid(ip->controls2) / 2 - wid(ip->button1) - 5,
+	xv_set(ip->button1, XV_X, wid(ip->controls2) / 2 - wid(ip->button1) - 25,
 		NULL);
 
 	ip->button8 = xv_create(ip->controls2, PANEL_BUTTON,
 		XV_HELP_DATA, "workman:button8",
-		XV_X, rig(ip->button1) + 10,
+		XV_X, rig(ip->button1) + 5,
 		XV_Y, top(ip->button1),
 		PANEL_LABEL_STRING, "Reset",
 		PANEL_NOTIFY_PROC, cdinfo_reset,
 		NULL);
-
+#if CDDB_IN_UI
+	ip->cddb = xv_create(ip->controls2, PANEL_BUTTON,
+		XV_HELP_DATA, "workman:cddb",
+		XV_X, rig(ip->button8) + 5,
+		XV_Y, top(ip->button8),
+		PANEL_LABEL_STRING, "CDDB",
+		PANEL_NOTIFY_PROC, connect_cddb,
+		NULL);
+#endif
+		
 	ip->buttonpl = xv_create(ip->controls2, PANEL_BUTTON,
 		XV_HELP_DATA, "workman:buttonpl",
 		XV_Y, top(ip->button1),
@@ -634,9 +699,7 @@ popup1_objects_init(ip, owner)
  * Create the About window and its contents.
  */
 about_objects *
-about_objects_init(ip, owner)
-	about_objects	*ip;
-	Xv_opaque		owner;
+about_objects_init( about_objects *ip, Xv_opaque owner ) 
 {
 	Panel_item	item;
 	int		space;
@@ -667,8 +730,9 @@ about_objects_init(ip, owner)
 
 	ip->message1 = xv_create(ip->controls3, PANEL_MESSAGE,
 		XV_Y, 8,
-		PANEL_LABEL_STRING, "WorkMan 1.3a",
+		PANEL_LABEL_STRING, WORKMAN_VERSION_STRING,
 		PANEL_LABEL_BOLD, TRUE, NULL);
+
 	ip->message2 = xv_create(ip->controls3, PANEL_MESSAGE,
 		PANEL_LABEL_STRING, "\"The Civilized CD Player\"",
 		PANEL_LABEL_BOLD, FALSE, NULL);
@@ -701,13 +765,13 @@ about_objects_init(ip, owner)
 		PANEL_LABEL_STRING, "Written by Steven Grimm",
 		PANEL_LABEL_BOLD, TRUE, NULL);
 	ip->message4 = xv_create(ip->controls3, PANEL_MESSAGE,
-		PANEL_LABEL_STRING, "koreth@Hyperion.COM",
+		PANEL_LABEL_STRING, "Maintained by Dirk Foersterling",
 		PANEL_LABEL_BOLD, FALSE, NULL);
 	ip->message5 = xv_create(ip->controls3, PANEL_MESSAGE,
-		PANEL_LABEL_STRING, "Send mail if you'd like to be",
+		PANEL_LABEL_STRING, "Read the file CONTACT for",
 		PANEL_LABEL_BOLD, FALSE, NULL);
 	ip->message6 = xv_create(ip->controls3, PANEL_MESSAGE,
-		PANEL_LABEL_STRING, "notified of future revisions!",
+		PANEL_LABEL_STRING, "contact information.",
 		PANEL_LABEL_BOLD, FALSE, NULL);
 
 	ip->drive = xv_create(ip->controls3, PANEL_MESSAGE,
@@ -722,7 +786,7 @@ about_objects_init(ip, owner)
 			space = wid(item);
 	PANEL_END_EACH;
 
-	xv_set(ip->drive, PANEL_LABEL_STRING, "Unknown drive type", NULL);
+/*	xv_set(ip->drive, PANEL_LABEL_STRING, "Unknown drive type", NULL); */
 
 	space += 10;
 	xv_set(ip->controls3, XV_WIDTH, space, NULL);
@@ -748,17 +812,17 @@ about_objects_init(ip, owner)
 	window_fit(ip->about);
 
 	return (ip);
-}
+} /* about_objects_init */ 
 
 /*
  * We've discovered the drive type, so update the About popup.
  */
 void
-about_set_drivetype(vendor, model, rev)
-	char	*vendor, *model, *rev;
+about_set_drivetype(unsigned char *vendor, unsigned char *model,
+                    unsigned char *rev )
 {
 	extern about_objects    *Workman_about;
-	char			drivetype[21];
+	char			drivetype[200];
 
 	sprintf(drivetype, "%s %s", vendor, model);
 	xv_set(Workman_about->drive, PANEL_LABEL_STRING, drivetype, NULL);
@@ -766,7 +830,9 @@ about_set_drivetype(vendor, model, rev)
 }
 
 /*
- * Create the Goodies window and its contents.
+ * Create the Goodies window and its contents.Note that the CDDA UI
+ * elements get created no matter what; the only thing that might not be
+ * enabled is the button to activate them.
  */
 goodies_objects *
 goodies_objects_init(ip, owner)
@@ -776,6 +842,11 @@ goodies_objects_init(ip, owner)
 	Panel_item	item;
 	Xv_opaque	phones_image;
 #include "bitmaps/phones0"
+#ifdef BUILD_CDDA
+#  include "bitmaps/rev_play"
+#  include "bitmaps/play.button.small"
+        Xv_opaque style_image0, style_image2;
+#endif
 	int		space;
 
 	if ((ip = (goodies_objects *) calloc(1,
@@ -797,7 +868,7 @@ goodies_objects_init(ip, owner)
 		XV_HELP_DATA, "workman:controls4",
 		XV_X, 0,
 		XV_Y, 0,
-		WIN_BORDER, FALSE,
+		WIN_BORDER, TRUE,
 		NULL);
 
 	ip->balance = xv_create(ip->controls4, PANEL_SLIDER,
@@ -829,6 +900,16 @@ goodies_objects_init(ip, owner)
 		PANEL_LABEL_BOLD, TRUE,
 		NULL);
 
+#ifdef BUILD_CDDA /* { */
+	ip->expand = xv_create(ip->controls4, PANEL_BUTTON,
+		XV_HELP_DATA, "workman:expand",
+		PANEL_LABEL_STRING, "+",
+		PANEL_NOTIFY_PROC, expand_goodies,
+		XV_SHOW, FALSE,
+		XV_Y, 6,
+		NULL);
+#endif /* } */
+
 	ip->timemode_track = xv_create(ip->controls4, PANEL_CHOICE,
 		XV_HELP_DATA, "workman:timemode",
 		PANEL_CHOICE_NROWS, 1,
@@ -929,6 +1010,103 @@ goodies_objects_init(ip, owner)
 		NULL);
 	put_down(ip->indexscan, ip->split, basic_spacing);
 
+#if CDDB_IN_UI
+	cddb_struct2cur();
+	
+	ip->cddb_protocol = xv_create(ip->controls4, PANEL_CHOICE,
+		XV_HELP_DATA, "workman:cddb_protocol",
+		XV_X, 10,
+		PANEL_CHOICE_NROWS, 1,
+		PANEL_LAYOUT, PANEL_HORIZONTAL,
+		PANEL_CHOOSE_NONE, FALSE,
+		PANEL_LABEL_STRING, "Use CDDB:",
+		PANEL_CHOICE_STRINGS,
+			"off",
+			"cddbp",
+			"http",
+			"proxy",	       
+			NULL,
+		PANEL_VALUE, cur_cddb_protocol,
+		PANEL_NOTIFY_PROC, cddb_select,
+		NULL);
+/*
+	put_down(ip->cddb_protocol, ip->phones, basic_spacing);
+*/
+	put_down(ip->cddb_protocol, ip->indexscan, basic_spacing);
+	
+	ip->cddb_server = xv_create(ip->controls4, PANEL_TEXT,
+		XV_HELP_DATA, "workman:cddb_server",
+		XV_X, 10,
+		PANEL_VALUE_DISPLAY_LENGTH, 20,
+		PANEL_VALUE_STORED_LENGTH, 80,
+		PANEL_LABEL_STRING, "CDDB server:",
+		PANEL_LAYOUT, PANEL_HORIZONTAL,
+		PANEL_READ_ONLY, FALSE,
+		PANEL_VALUE, cur_cddb_server,
+		PANEL_NOTIFY_PROC, update_cddb_goodies,
+		NULL);
+	put_down(ip->cddb_server, ip->cddb_protocol, basic_spacing);
+
+	ip->cddb_mailadress = xv_create(ip->controls4, PANEL_TEXT,
+		XV_HELP_DATA, "workman:cddb_mailadress",
+		XV_X, 10,
+		PANEL_VALUE_DISPLAY_LENGTH, 20,
+		PANEL_VALUE_STORED_LENGTH, 80,
+		PANEL_LABEL_STRING, "Email adress:",
+		PANEL_LAYOUT, PANEL_HORIZONTAL,
+		PANEL_READ_ONLY, FALSE,
+		PANEL_VALUE, cur_cddb_mail_adress,
+		PANEL_NOTIFY_PROC, update_cddb_goodies,
+		NULL);
+	put_down(ip->cddb_mailadress, ip->cddb_server, basic_spacing);
+
+	ip->cddb_pathtocgi = xv_create(ip->controls4, PANEL_TEXT,
+		XV_HELP_DATA, "workman:cddb_pathtocgi",
+		XV_X, 10,
+		PANEL_VALUE_DISPLAY_LENGTH, 20,
+		PANEL_VALUE_STORED_LENGTH, 80,
+		PANEL_LABEL_STRING, "Path to cgi:",
+		PANEL_LAYOUT, PANEL_HORIZONTAL,
+		PANEL_READ_ONLY, FALSE,
+		PANEL_VALUE, cur_cddb_path_to_cgi,
+		PANEL_NOTIFY_PROC, update_cddb_goodies,
+		NULL);
+	put_down(ip->cddb_pathtocgi, ip->cddb_mailadress, basic_spacing);
+
+	ip->cddb_proxy = xv_create(ip->controls4, PANEL_TEXT,
+		XV_HELP_DATA, "workman:cddb_proxy",
+		XV_X, 10,
+		PANEL_VALUE_DISPLAY_LENGTH, 20,
+		PANEL_VALUE_STORED_LENGTH, 80,
+		PANEL_LABEL_STRING, "Proxy server:",
+		PANEL_LAYOUT, PANEL_HORIZONTAL,
+		PANEL_READ_ONLY, FALSE,
+		PANEL_VALUE, cur_cddb_proxy_server,
+		PANEL_NOTIFY_PROC, update_cddb_goodies,
+		NULL);
+	put_down(ip->cddb_proxy, ip->cddb_pathtocgi, basic_spacing);
+
+#define LONG	ip->cddb_mailadress
+#define SHORT1	ip->cddb_pathtocgi
+#define SHORT2	ip->cddb_proxy
+#define SHORT3	ip->cddb_server
+	
+	xv_set(SHORT1, PANEL_VALUE_X, (int) xv_get(LONG,
+		PANEL_VALUE_X), NULL);
+
+	xv_set(SHORT2, PANEL_VALUE_X, (int) xv_get(LONG,
+		PANEL_VALUE_X), NULL);
+
+	xv_set(SHORT3, PANEL_VALUE_X, (int) xv_get(LONG,
+		PANEL_VALUE_X), NULL);
+
+#undef LONG
+#undef SHORT1
+#undef SHORT2
+#undef SHORT3
+	
+#endif
+	
 	space = wid(ip->a) + wid(ip->alabel) + basic_spacing;
 	PANEL_EACH_ITEM(ip->controls4, item)
 		xv_set(item, XV_KEY_DATA, INSTANCE, ip, NULL);
@@ -938,9 +1116,18 @@ goodies_objects_init(ip, owner)
 
 	space += 10;
 	xv_set(ip->controls4, XV_WIDTH, space, XV_HEIGHT,
+#if CDDB_IN_UI
+		bot(ip->cddb_proxy) + 5, NULL);
+#else
 		bot(ip->indexscan) + 5, NULL);
+#endif
+
 	window_fit(ip->goodies);
 
+#ifdef BUILD_CDDA
+       xv_set(ip->expand, XV_X, space - wid(ip->expand) - basic_spacing, NULL);
+#endif
+	
 	center_x(ip->timemode_cd, ip->controls4);
 	center_x(ip->timemode_track, ip->controls4);
 	center_x(ip->playnewcds, ip->controls4);
@@ -961,6 +1148,177 @@ goodies_objects_init(ip, owner)
 
 	center_x(ip->indexscan, ip->controls4);
 
+	ip->smallwidth = xv_get(ip->goodies, XV_WIDTH);
+	ip->bigwidth = ip->smallwidth;
+
+#ifdef BUILD_CDDA /* { */
+	/*
+	 * The CDDA elements appear to the right of the regular controls, in
+	 * a completely different panel.  The window expands to show them if
+	 * appropriate.
+	 */
+	ip->cdda = xv_create(ip->goodies, PANEL,
+		XV_KEY_DATA, INSTANCE, ip,
+		XV_HELP_DATA, "workman:cdda",
+		XV_X, 0,
+		XV_Y, 0,
+		XV_HEIGHT, hei(ip->controls4),
+		WIN_BORDER, TRUE,
+		NULL);
+
+	ip->title = xv_create(ip->cdda, PANEL_TOGGLE,
+		XV_Y, 6,
+		PANEL_CHOICE_NCOLS, 1,
+		PANEL_CHOICE_STRING, 0, "Enable Digital Audio",
+                PANEL_VALUE, 0,
+		PANEL_NOTIFY_PROC, change_cdda,
+		NULL);
+
+	style_image0 = xv_create(XV_NULL, SERVER_IMAGE,
+		SERVER_IMAGE_DEPTH, 1,
+		SERVER_IMAGE_X_BITS, rev_play_bits,
+		XV_WIDTH, 16,
+		XV_HEIGHT, 16,
+		NULL);
+
+	style_image2 = xv_create(XV_NULL, SERVER_IMAGE,
+		SERVER_IMAGE_DEPTH, 1,
+		SERVER_IMAGE_X_BITS, play_button_small_bits,
+		XV_WIDTH, 16,
+		XV_HEIGHT, 16,
+		NULL);
+
+	ip->playtitle = xv_create(ip->cdda, PANEL_MESSAGE,
+		PANEL_LABEL_BOLD, TRUE,
+		PANEL_LABEL_STRING, "Playback speed:",
+		NULL);
+	put_down(ip->playtitle, ip->title, basic_spacing * 3);
+
+	ip->playspeed = xv_create(ip->cdda, PANEL_SLIDER,
+		PANEL_DIRECTION, PANEL_HORIZONTAL,
+		PANEL_MIN_VALUE, 0,
+		PANEL_MAX_VALUE, 255,
+		PANEL_VALUE, 128,
+		PANEL_MIN_VALUE_STRING, "Half",
+		PANEL_MAX_VALUE_STRING, "4x",
+		PANEL_SHOW_VALUE, FALSE,
+		PANEL_SHOW_RANGE, TRUE,
+		PANEL_SLIDER_END_BOXES, FALSE,
+		PANEL_TICKS, 3,
+		PANEL_SLIDER_WIDTH, 50,
+		XV_HELP_DATA, "workman:playstyle",
+		PANEL_NOTIFY_PROC, change_speed,
+		PANEL_INACTIVE, TRUE,
+		NULL);
+	put_down(ip->playspeed, ip->playtitle, basic_spacing);
+
+	ip->playstyle = xv_create(ip->cdda, PANEL_CHOICE,
+		XV_HELP_DATA, "workman:playstyle",
+		PANEL_CHOICE_NROWS, 1,
+		PANEL_LAYOUT, PANEL_VERTICAL,
+		PANEL_CHOOSE_NONE, FALSE,
+		PANEL_CHOICE_IMAGES,
+			style_image0,
+			style_image2,
+			NULL,
+		PANEL_VALUE, 1,
+		PANEL_NOTIFY_PROC, change_style,
+		PANEL_INACTIVE, TRUE,
+		NULL);
+
+	put_right(ip->playstyle, ip->playspeed, basic_spacing * 2);
+	space = bot(ip->playspeed) + top(ip->playtitle) - hei(ip->playstyle);
+	xv_set(ip->playstyle,
+		XV_Y, space / 2,
+		NULL);
+
+        ip->range = xv_create(ip->cdda, PANEL_TOGGLE,
+		XV_HELP_DATA, "workman:range",
+		PANEL_CHOICE_NCOLS, 1,
+		PANEL_CHOICE_STRING, 0, "Flatten Volume Levels",
+                PANEL_VALUE, 0,
+		PANEL_NOTIFY_PROC, change_range,
+		PANEL_INACTIVE, TRUE,
+                NULL);
+	put_down(ip->range, ip->playspeed, basic_spacing * 2);
+
+        ip->tone = xv_create(ip->cdda, PANEL_SLIDER,
+		XV_HELP_DATA, "workman:tone",
+                PANEL_SLIDER_WIDTH, 50,
+                PANEL_TICKS, 0,
+                PANEL_DIRECTION, PANEL_HORIZONTAL,
+                PANEL_SLIDER_END_BOXES, FALSE,
+                PANEL_SHOW_RANGE, TRUE,
+                PANEL_SHOW_VALUE, FALSE,
+                PANEL_MIN_VALUE, 0,
+                PANEL_MAX_VALUE, 100,
+                PANEL_MIN_VALUE_STRING, "Bass",
+                PANEL_MAX_VALUE_STRING, "Treble",
+                PANEL_VALUE, 50,
+		PANEL_INACTIVE, TRUE,
+                NULL);
+	put_down(ip->tone, ip->range, basic_spacing * 3);
+
+	ip->filename = xv_create(ip->cdda, PANEL_TEXT,
+		XV_HELP_DATA, "workman:filename",
+		PANEL_LABEL_STRING, "Save audio to file:",
+		PANEL_LAYOUT, PANEL_VERTICAL,
+		PANEL_VALUE_DISPLAY_LENGTH, 16,
+		PANEL_VALUE_STORED_LENGTH, 200,
+		PANEL_READ_ONLY, FALSE,
+		PANEL_NOTIFY_PROC, change_filename,
+		PANEL_NOTIFY_LEVEL, PANEL_ALL,
+		PANEL_INACTIVE, TRUE,
+		NULL);
+	put_down(ip->filename, ip->tone, basic_spacing * 3);
+
+	ip->save = xv_create(ip->cdda, PANEL_TOGGLE,
+		XV_HELP_DATA, "workman:save",
+		PANEL_CHOICE_NCOLS, 1,
+		PANEL_CHOICE_STRING, 0, "Save",
+		PANEL_NOTIFY_PROC, change_save,
+		PANEL_VALUE, 0,
+		PANEL_INACTIVE, TRUE,
+		NULL);
+	center_y(ip->save, ip->filename);
+	put_right(ip->save, ip->filename, basic_spacing * 2);
+
+	space = wid(ip->playstyle) + wid(ip->playspeed) + basic_spacing;
+
+	PANEL_EACH_ITEM(ip->cdda, item)
+		xv_set(item, XV_KEY_DATA, INSTANCE, ip, NULL);
+		if (wid(item) > space)
+			space = wid(item);
+	PANEL_END_EACH;
+
+	if (wid(ip->save) + wid(ip->filename) + basic_spacing * 2 > space)
+		space = wid(ip->save) + wid(ip->filename) + basic_spacing * 2;
+
+	space += 25;
+	xv_set(ip->cdda, XV_WIDTH, space, NULL);
+	window_fit(ip->goodies);
+
+	center_x(ip->title, ip->cdda);
+	center_x(ip->tone, ip->cdda);
+	center_x(ip->save, ip->cdda);
+	center_x(ip->filename, ip->cdda);
+	center_x(ip->range, ip->cdda);
+	center_x(ip->tone, ip->cdda);
+
+	space = wid(ip->playstyle) + wid(ip->playspeed) + basic_spacing * 2;
+	xv_set(ip->playspeed, XV_X, (wid(ip->cdda) - space) / 2, NULL);
+	put_right(ip->playstyle, ip->playspeed, basic_spacing * 2);
+	xv_set(ip->playtitle, XV_X, lef(ip->playspeed), NULL);
+
+	space = wid(ip->save) + wid(ip->filename) + basic_spacing * 2;
+	xv_set(ip->filename, XV_X, (wid(ip->cdda) - space) / 2, NULL);
+	put_right(ip->save, ip->filename, basic_spacing * 2);
+
+	ip->bigwidth += wid(ip->cdda);
+	xv_set(ip->cdda, XV_X, wid(ip->controls4), NULL);
+
+#endif /* } */
+
 	return (ip);
 }
 
@@ -1068,7 +1426,7 @@ plpopup_objects_init(ip, owner)
 		XV_X, lef(ip->playlists),
 		PANEL_LIST_WIDTH, rig(ip->listname) - lef(ip->playlists),
 		PANEL_LIST_DISPLAY_ROWS, 5,
-		PANEL_LIST_TITLE, "Playlist (add tracks from popup menu)",
+		PANEL_LIST_TITLE, "Playlist (use right mouse button)",
 		PANEL_LAYOUT, PANEL_VERTICAL,
 		PANEL_READ_ONLY, TRUE,
 		PANEL_CHOOSE_ONE, TRUE,
@@ -1100,39 +1458,3 @@ plpopup_objects_init(ip, owner)
 
 	return (ip);
 }
-
-/*
- * Create the database preferences window and its contents.
-db_prefs_objects *
-db_prefs_init(ip, owner)
-	db_prefs_objects	*ip;
-	Xv_opaque		owner;
-{
-	if ((ip = (db_prefs_objects *) calloc(1,
-				sizeof(db_prefs_objects))) == NULL)
-		return (NULL);
-	
-	ip->window = xv_create(owner, FRAME_CMD,
-		XV_KEY_DATA, INSTANCE, ip,
-		XV_LABEL, "Databases",
-		XV_SHOW, FALSE,
-		FRAME_SHOW_FOOTER, FALSE,
-		FRAME_SHOW_RESIZE_CORNER, TRUE,
-		FRAME_CMD_PUSHPIN_IN, FALSE,
-		NULL);
-	
-	ip->panel = xv_get(ip->window, FRAME_CMD_PANEL);
-	xv_set(ip->panel,
-		XV_KEY_DATA, INSTANCE, ip,
-		XV_HELP_DATA, "workman:dbprefs-panel",
-		XV_X, 0,
-		XV_Y, 0,
-		WIN_BORDER, FALSE,
-		NULL);
-	
-	ip->filelist = xv_create(ip->panel, PANEL_LIST,
-		XV_HELP_DATA, "workman:dbprefs-filelist",
-		XV_X, 6,
-		XV_Y, 8,
-		PANEL_LIST_WIDTH, basic_spacing * 40
- */
