@@ -1,5 +1,5 @@
 /*
- * $Id: workman_stubs.c,v 1.15 1997/12/20 10:15:52 dirk Exp $
+ * $Id: workman_stubs.c,v 1.7 1999/10/27 05:48:06 dirk Exp $
  *
  * This file is part of WorkMan, the civilized CD player program
  * (c) 1991-1997 by Steven Grimm (original author)
@@ -20,14 +20,18 @@
  * workman_stubs.c - Notify and event callback function stubs.
  */
 
-static char workman_stubs_id[] = "$Id: workman_stubs.c,v 1.15 1997/12/20 10:15:52 dirk Exp $";
+static char workman_stubs_id[] = "$Id: workman_stubs.c,v 1.7 1999/10/27 05:48:06 dirk Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <unistd.h>
 #include <limits.h>
+#ifndef sun
+#include <getopt.h>
+#endif
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -43,6 +47,9 @@ static char workman_stubs_id[] = "$Id: workman_stubs.c,v 1.15 1997/12/20 10:15:5
 #include "struct.h"
 #include "config.h"
 #include "proto.h"
+#ifdef CDDB_IN_UI
+#include "wm_cddb.h"
+#endif
 
 /*
  * local prototypes
@@ -68,7 +75,7 @@ char 	       *empty = "";
 extern char    *cd_device;
 
 Rect	*track_rect = NULL;
-Xv_Notice wannasave, mountedfs;
+Xv_Notice wannasave, mountedfs, noeject;
 int	confirmsave;
 int	add_height, small_height;
 int	min_lines = -1;
@@ -115,6 +122,9 @@ int dismiss_button = 1;
 #else
 int dismiss_button = 0;
 #endif
+int iscdclosed = 1;
+
+char *cddaslave_path = NULL;
 
 Attr_attribute	INSTANCE;
 
@@ -128,13 +138,15 @@ main( int argc, char **argv )
 {
 	int		c;
 	int		fd;
+	uid_t		userid;
+	char		*slash;
         extern char     *optarg, *rcfile, *dbfiles;
-	extern int	keep_open;
-	char		pidfname[78];  /* 65 bytes hostname, 13 bytes path */
+	char		pidfname[83];  /* 65 bytes hostname, 13 bytes path, 5 Bytes UID */
 	struct utsname  unamebuf;      /* for uname() */
  
+	userid = getuid();
 #if defined(SVR4) && !defined(__sony_news) /* { */
-        if (setreuid(-1, getuid()) < 0)
+        if (setreuid(-1, userid) < 0)
         {
                 perror("setreuid");
                 exit(-1);
@@ -150,11 +162,70 @@ main( int argc, char **argv )
 	  {
 	  	strcat( unamebuf.nodename, "localhost" );
 	  }
-	sprintf( pidfname, "/tmp/.wm-pid-%s", unamebuf.nodename);
+	sprintf( pidfname, "/tmp/.wm-pid-%s-%05d", unamebuf.nodename, userid);
 
 	if( strlen(pidfile) == 0 )
             pidfile = pidfname;
+
+	/*
+	 * Initialize XView.  First strip out -geometry or -Ws from the
+	 * command line args; XView won't let us resize the window on the
+	 * fly if the size is user-specified.  (Which will only be the
+	 * case because the user saved his desktop setup, typically.)
+	 */
+	strip_geometry(&argc, argv);
+
+        /* added XV_USE_LOCALE to prevent charset problems */
+	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, XV_USE_LOCALE, TRUE, NULL);
 	
+#ifdef __sony_news
+	XSetErrorHandler(&error_handler);
+#endif
+	INSTANCE = xv_unique_key();
+
+	/*
+	 * Initialize the list of database locations.  Precedence is
+	 * command-line then resource then environment.
+	 */
+	rcfile = getenv("WORKMANRC");
+	rcfile = defaults_get_string("workman.db.personal",
+		"Workman.Db.Personal", rcfile);
+	if (rcfile)
+		rcfile = WMstrdup(rcfile);
+	dbfiles = getenv("WORKMANDB");
+	dbfiles = defaults_get_string("workman.db.shared",
+		"Workman.Db.Shared", dbfiles);
+	if (dbfiles)
+		dbfiles = WMstrdup(dbfiles);
+
+	reverse_threshold = defaults_get_integer("workman.reverseThreshold",
+		"Workman.ReverseThreshold", reverse_threshold);
+
+	
+	/* The cddaslave program is probably located alongside WorkMan...  */
+	slash = strrchr(argv[0], '/');
+	if (slash)
+	{
+		*slash = '\0';
+		cddaslave_path = malloc(strlen(argv[0]) + sizeof("/cddaslave"));
+		if (cddaslave_path)
+			sprintf(cddaslave_path, "%s/cddaslave", argv[0]);
+		*slash = '/';
+	}
+	else
+		cddaslave_path = WMstrdup("cddaslave");
+
+	/* But it could be specified by an X resource. */
+	slash = cddaslave_path;
+	cddaslave_path = defaults_get_string("workman.cddaslave",
+		"Workman.Cddaslave", cddaslave_path);
+	if (slash != cddaslave_path)
+	{
+	        free(slash);
+	        cddaslave_path = WMstrdup(cddaslave_path);
+	}
+
+
 	while ((c = getopt(argc, argv, "s:p:dc:ol:eXnbV:hCD:vf:?")) != EOF)
 		switch (c) {
 		case 'p':
@@ -177,7 +248,7 @@ main( int argc, char **argv )
 			cd_device = optarg;
 			break;
 		case 'o':
-			keep_open = 0;
+			/* was: don't keep device open */
 			break;
 		case 'l':
 			min_lines = atoi(optarg);
@@ -227,7 +298,7 @@ See the file COPYING for details.\n", WORKMAN_VERSION_STRING, __DATE__ );
 "  -h  and  -?  display this message\n\
   -l  leave room for at least N lines of track title\n\
   -n  don't use file locking when updating database (dangerous)\n\
-  -o  don't run background job to keep device open (Solaris 2.0/2.1)\n\
+  -o  (ignored for backwards compatibility) \n\
   -p  write process ID to another file (default = %s)\n\
   -s  send command to running WorkMan\n\
   -v  display version information\n", pidfname);
@@ -237,38 +308,12 @@ See the file COPYING for details.\n", WORKMAN_VERSION_STRING, __DATE__ );
   -V  minimum volume setting (default = %d)\n\
   -X  exit when CD is ejected\n", min_volume);
 			fprintf( stderr,
-"\nWorkMan is free software under GNU General Public License.\n\
-See the file COPYING for details.\n");
+"\nThis is %s\nWorkMan is free software under GNU General Public License.\n\
+See the file COPYING for details.\n", WORKMAN_VERSION_STRING);
 
 			exit(1);
 		}
 
-	/*
-	 * Initialize XView.
-	 */
-	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, XV_USE_LOCALE, TRUE, NULL);
-#ifdef __sony_news
-	XSetErrorHandler(&error_handler);
-#endif
-	INSTANCE = xv_unique_key();
-
-	/*
-	 * Initialize the list of database locations.  Precedence is
-	 * command-line then resource then environment.
-	 */
-	rcfile = getenv("WORKMANRC");
-	rcfile = defaults_get_string("workman.db.personal",
-		"Workman.Db.Personal", rcfile);
-	if (rcfile)
-		rcfile = WMstrdup(rcfile);
-	dbfiles = getenv("WORKMANDB");
-	dbfiles = defaults_get_string("workman.db.shared",
-		"Workman.Db.Shared", dbfiles);
-	if (dbfiles)
-		dbfiles = WMstrdup(dbfiles);
-
-	reverse_threshold = defaults_get_integer("workman.reverseThreshold",
-		"Workman.ReverseThreshold", reverse_threshold);
 
 	/*
 	 * Initialize the user-interface components.
@@ -282,11 +327,22 @@ See the file COPYING for details.\n");
 	/* Cook the database filenames, or construct them if none were given. */
 	split_workmandb();
 
+/*
+ * Analogous to 1.4b3, remove keep_cd_open()
+ *
 	if (keep_open)
 		if (fork() == 0)
 			keep_cd_open();
 		else
 			wait(NULL);
+*/
+
+	/*
+	 * Load default settings from .workmanrc.
+	 */
+	load_settings();
+	if (cur_stopmode == -1)
+		cur_stopmode = 0;
 
 	/*
 	 * Fill up the PID-file.  Have to do this in a bit of a roundabout
@@ -371,8 +427,13 @@ See the file COPYING for details.\n");
 			NULL);
 		xv_set(button, XV_X, ((int) xv_get(Workman_about->about,
 			XV_WIDTH) - (int) xv_get(button, XV_WIDTH)) / 2,
+#if CDDB_IN_UI
+			XV_Y, (int) xv_get(Workman_goodies->cddb_proxy, XV_Y) +
+			(int) xv_get(Workman_goodies->cddb_proxy, XV_HEIGHT) + 5,
+#else
 			XV_Y, (int) xv_get(Workman_about->drive, XV_Y) +
 			(int) xv_get(Workman_about->drive, XV_HEIGHT) + 5,
+#endif
 			NULL);
 		button = xv_create(Workman_goodies->controls4, PANEL_BUTTON,
 			PANEL_LABEL_STRING, "Dismiss", PANEL_NOTIFY_PROC,
@@ -396,8 +457,14 @@ See the file COPYING for details.\n");
 	xv_set(Workman_goodies->abrepeat, PANEL_INACTIVE, TRUE, NULL);
 
 	/* Initialize some stuff Guide won't do. */
+
+/*
+ * this is removed as in 1.4b3
+ *
 	xv_set(Workman_popup1->defaultvolume, PANEL_NOTIFY_LEVEL, PANEL_ALL,
 		NULL);
+*/
+
 	xv_set(Workman_window1->songpos, PANEL_NOTIFY_LEVEL, PANEL_ALL,
 		PANEL_JUMP_DELTA, 5, NULL);
 	xv_set(Workman_about->about, FRAME_CMD_PUSHPIN_IN, FALSE, NULL);
@@ -455,6 +522,13 @@ See the file COPYING for details.\n");
 		"Please run 'umount' before ejecting",
 		"or nasty things may happen.", NULL, NOTICE_BUTTON,
 		"Okay", 0, NULL);
+	noeject = xv_create(Workman_window1->window1, NOTICE,
+		NOTICE_MESSAGE_STRINGS, "WorkMan warning!", "",
+		"The CD-ROM device could not be opened for ejecting the CD.",
+		"The driver software for your drive may lack support for handling",
+		"empty CD-ROM drives. Please insert a CD manually now and confirm",
+		"by pressing Okay below.", NULL, NOTICE_BUTTON,
+		"Okay", 0, NULL);
 
 	/*
 	 * Turn control over to XView.
@@ -540,6 +614,43 @@ handle_timer( Notify_client c, int w )
 	return (NOTIFY_DONE);
 }
 
+/*
+ * Ejecting the CD-ROM putting out error notices.
+ */
+int eject_cd_stub( window1_objects *ip )
+{
+		int status = 0;
+		
+		status = eject_cd();
+		switch ( status ) {
+		case 0:
+			setup_itimer(ip, 5);
+			kill_stats(ip);
+			/*
+			 * need to call wipe_cdinfo() because the one in
+			 * handle_timer() won't be used with -e -e
+			 */
+			wipe_cdinfo();
+			iscdclosed = 0;
+			break;
+		case 1: /* Device could not be opened */
+			/* put a warning and pretend we succeeded. */
+			xv_set(ip->mode, PANEL_VALUE, 5, NULL); 
+			xv_set(noeject, XV_SHOW, TRUE, NULL);
+			setup_itimer(ip, 5);
+			kill_stats(ip);
+			wipe_cdinfo();
+			iscdclosed = 0;
+			status = 0;
+			break;
+		case 2:
+			xv_set(ip->mode, PANEL_VALUE, 4, NULL);
+			xv_set(mountedfs, XV_SHOW, TRUE, NULL);
+			break;
+		}
+	return status;
+} /* eject_cd_stub() */
+
 /*
  * The CD is in the drive.  Update all the necessary screen elements and handle
  * end-of-track and so forth.  Returns 1 if handle_timer() should get the CD
@@ -554,7 +665,7 @@ update_everything( window1_objects *ip )
 donewithcd:
 		if (xv_get(Workman_goodies->abrepeat, PANEL_VALUE))
 		{
-			play_chunk(mark_a, mark_b);
+			play_chunk(mark_a, mark_b, mark_a);
 			return (1);
 		}
 		if (was_repeating)
@@ -562,11 +673,16 @@ donewithcd:
 			was_repeating = 0;
 			play_chunk(mark_b, cur_lasttrack >= cur_ntracks ?
 				(cd->length - 1) * 75 :
-				cd->trk[cur_lasttrack].start-1);
+				cd->trk[cur_lasttrack].start-1,
+				cd->trk[cur_track-1].start);
 			return (1);
 		}
 
-		play_next_entry();
+#ifdef BUILD_CDDA
+		play_next_entry(xv_get(Workman_goodies->playstyle,PANEL_VALUE));
+#else
+		play_next_entry(0);
+#endif
 		if (cd_status() != 1)
 			return (1);
 		if (cur_cdmode == STOPPED)	/* Done with CD */
@@ -578,18 +694,27 @@ donewithcd:
 			case 1:
 				make_playlist(xv_get(ip->shuffle,
 					PANEL_VALUE), 0);
-				play_next_entry();
+#ifdef BUILD_CDDA
+				play_next_entry(xv_get(
+						Workman_goodies->playstyle,
+						PANEL_VALUE));
+#else
+				play_next_entry(0);
+#endif						
 				break;
 			case 2:
 				keep_settings(ip);
-				if (info_modified)
+				if (info_modified &&
+					xv_get(Workman_popup1->button1,
+					XV_SHOW))
+				
 				{
 					xv_set(wannasave, XV_SHOW, TRUE, NULL);
 					if (confirmsave)
 						save_config( (int)NULL, NULL );
 					info_modified = 0;
 				}
-				if (eject_cd() == 0)
+				if (eject_cd_stub(ip) == 0)
 				{
 					setup_itimer(ip, 5);
 					kill_stats(ip);
@@ -732,7 +857,12 @@ make_initial_playlist( int playmode )
 	else
 	{
 		make_playlist(get_playmode(), 0);
-		play_next_entry();
+#ifdef BUILD_CDDA
+		play_next_entry(xv_get(Workman_goodies->playstyle,
+					PANEL_VALUE));
+#else
+		play_next_entry(0);
+#endif 
 	}
 }
 
@@ -752,6 +882,7 @@ setup_itimer( window1_objects *ip, int interval )
 		&it, NULL);
 }
 
+
 /*
  * Notify callback function for `mode'.
  */
@@ -763,6 +894,19 @@ change_mode( Panel_item item, int value, Event *event )
 
 	if (cur_cdmode == EJECTED && dont_retry)
 		handle_timer( (int)NULL, (int)NULL );
+	if (cur_cdmode == EJECTED && value == 5 && iscdclosed && !eject_cd_stub(ip))
+	{
+		iscdclosed = 0;
+		return;
+	}
+	if (cur_cdmode == EJECTED && value == 5)
+	{
+		iscdclosed = 1;
+		if (cd_closetray())
+			new_cd_inserted(ip);
+		return;
+	}
+		
 	if (cur_cdmode == EJECTED || (cur_track == -1 && (value == 0 || value == 2)))
 	{
 		xv_set(ip->mode, PANEL_VALUE, cur_cdmode, NULL);
@@ -779,7 +923,13 @@ change_mode( Panel_item item, int value, Event *event )
 				time_wanted = 0;
 			else
 			{
-				play_prev_track();
+#ifdef BUILD_CDDA
+				play_prev_track(xv_get(
+						Workman_goodies->playstyle,
+						PANEL_VALUE));
+#else
+				play_prev_track(0);
+#endif
 				cd_status();
 			}
 		}
@@ -814,8 +964,10 @@ change_mode( Panel_item item, int value, Event *event )
 		}
 		if (cur_cdmode == STOPPED)
 		{
+
 			/* XXX should call make_initial_playlist() */
 			track = xv_get(ip->tracks, PANEL_VALUE) + 1;
+
 			playmode = xv_get(ip->shuffle, PANEL_VALUE);
 			if (playmode == 1)
 				make_playlist(1, track);
@@ -832,8 +984,14 @@ change_mode( Panel_item item, int value, Event *event )
 				}
 			}
 		}
+
 		if (cur_cdmode != PLAYING)
-			play_next_entry();
+#ifdef BUILD_CDDA
+			play_next_entry(xv_get(Workman_goodies->playstyle,
+						PANEL_VALUE));
+#else
+			play_next_entry(0);
+#endif
 		cd_status();
 
 		/* We're at the end of the previous track. */
@@ -847,7 +1005,12 @@ change_mode( Panel_item item, int value, Event *event )
 	case 2:		/* forward a track */
 		if (cur_cdmode == PLAYING)
 		{
-			play_next_track();
+#ifdef BUILD_CDDA
+			play_next_track(xv_get(Workman_goodies->playstyle,
+						PANEL_VALUE));
+#else
+			play_next_track(0);
+#endif
 			if (cur_cdmode == STOPPED)
 				goto stopped;
 			cd_status();
@@ -898,9 +1061,9 @@ stopped:
 		keep_settings(ip);
 
 #ifdef CAN_CLOSE
-		if ((info_modified) && (cur_cdmode != EJECTED))
+		if ((info_modified) && (cur_cdmode != EJECTED) && xv_get(Workman_popup1->button1, XV_SHOW))
 #else
-		if (info_modified)
+		if (info_modified && xv_get(Workman_popup1->button1, XV_SHOW))
 #endif
 		{
 			xv_set(wannasave, XV_SHOW, TRUE, NULL);
@@ -909,28 +1072,10 @@ stopped:
 			info_modified = 0;
 		}
 
-		switch ( eject_cd() ) {
-		case 0:
-			setup_itimer(ip, 5);
-			kill_stats(ip);
-			/*
-			 * need to call wipe_cdinfo() because the one in
-			 * handle_timer() won't be used with -e -e
-			 */
-			wipe_cdinfo();
-			break;
-		case 1:
-			xv_set(ip->mode, PANEL_VALUE, 4, NULL);
-			break;	/* XXX - should display an error popup */
-		case 2:
-			xv_set(ip->mode, PANEL_VALUE, 4, NULL);
-			xv_set(mountedfs, XV_SHOW, TRUE, NULL);
-			break;
-		}
-
+		eject_cd_stub(ip);
 		break;
 	}
-}
+} /* change_mode */
 
 /*
  * Notify callback function for `button2'.  Show the CD Info popup.
@@ -973,7 +1118,15 @@ show_cdinfo( Panel_item item, Event *event )
 		if ((int) xv_get(ip->window1, XV_Y) + cdi_height >
 							DisplayHeight(dpy, c))
 			xv_set(Workman_popup1->popup1, XV_Y,
-				DisplayHeight(dpy, c) - cdi_height - 28, NULL);
+				DisplayHeight(dpy, c) - cdi_height - 38, NULL);
+
+/* 
+ * fvwm does not always correctly return the window position. Thus we
+ * have to check that the Info window is not placed above the screen.
+ * Hack introduced by Debian maintainer Volker Ossenkopf.
+ */
+		else if ((int) xv_get(ip->window1, XV_Y) < 25)
+			xv_set(Workman_popup1->popup1, XV_Y, 0, NULL);
 		else
 			xv_set(Workman_popup1->popup1, XV_Y, xv_get(ip->
 				window1, XV_Y) - 25, NULL);
@@ -1002,6 +1155,90 @@ show_cdinfo( Panel_item item, Event *event )
 	}
 }
 
+#ifdef CDDB_IN_UI
+/*
+ * Notify callback function for `cddb'.
+ */
+void
+connect_cddb(Panel_item item, int value, Event *event)
+{
+	window1_objects *ip = Workman_window1;
+
+	kill_stats(ip);
+	cddb_request();
+        init_stats(ip);
+	show_stats(ip);
+/*        show_cdinfo(); */
+}
+
+/*
+ * Notify callback function for `cddb protocol'
+ */
+void
+cddb_select(Panel_item item, int value, Event *event)
+{
+	goodies_objects *gu = Workman_goodies;
+	popup1_objects	*pu = Workman_popup1;
+	window1_objects *ip = Workman_window1;
+
+/*
+	printf("ping:%d\n",value); 
+*/
+	cddb_struct2cur();
+	if(value >= 0)
+	{
+		cur_cddb_protocol = value;
+		xv_set(gu->cddb_server,     PANEL_INACTIVE, TRUE, NULL);
+		xv_set(gu->cddb_mailadress, PANEL_INACTIVE, TRUE, NULL);
+		xv_set(gu->cddb_pathtocgi,  PANEL_INACTIVE, TRUE, NULL);
+		xv_set(gu->cddb_proxy,      PANEL_INACTIVE, TRUE, NULL);
+		xv_set(pu->cddb,	    PANEL_INACTIVE, TRUE, NULL);
+		
+		switch (value)
+		{
+		 case 3:
+			xv_set(gu->cddb_proxy,
+			       PANEL_INACTIVE, FALSE, NULL);
+		 case 2:
+			xv_set(gu->cddb_pathtocgi,
+			       PANEL_INACTIVE, FALSE, NULL);
+		 case 1:
+			xv_set(gu->cddb_mailadress,
+			       PANEL_INACTIVE, FALSE, NULL);
+			xv_set(gu->cddb_server,
+			       PANEL_INACTIVE, FALSE, NULL);
+			xv_set(pu->cddb, PANEL_INACTIVE, FALSE, NULL);
+		 case 0:
+		 default:
+		}
+	}
+	xv_set(gu->cddb_protocol,   PANEL_VALUE, cur_cddb_protocol, NULL);
+	xv_set(gu->cddb_server,     PANEL_VALUE, cur_cddb_server, NULL);
+	xv_set(gu->cddb_mailadress, PANEL_VALUE, cur_cddb_mail_adress, NULL);
+	xv_set(gu->cddb_pathtocgi,  PANEL_VALUE, cur_cddb_path_to_cgi, NULL);
+	xv_set(gu->cddb_proxy,      PANEL_VALUE, cur_cddb_proxy_server, NULL);
+
+	cddb_cur2struct();
+}
+
+Panel_setting
+update_cddb_goodies(Panel_item item, Event *event)
+{
+	goodies_objects *gu = Workman_goodies;
+
+	if (event == NULL)
+		return ((Panel_setting)NULL);
+	
+	cur_cddb_server      =(char *)xv_get(gu->cddb_server,     PANEL_VALUE);
+	cur_cddb_mail_adress =(char *)xv_get(gu->cddb_mailadress, PANEL_VALUE);
+	cur_cddb_path_to_cgi =(char *)xv_get(gu->cddb_pathtocgi,  PANEL_VALUE);
+	cur_cddb_proxy_server=(char *)xv_get(gu->cddb_proxy,      PANEL_VALUE);
+	cddb_cur2struct();
+	return panel_text_notify(item, event);
+}
+
+#endif /* CDDB_IN_UI */
+
 /*
  * Notify callback function for `tracks'.
  */
@@ -1042,6 +1279,8 @@ change_track( Panel_item item, int value, Event *event )
 
 /*
  * Notify callback function for `songpos'.
+ * 'songpos' ist the slider for selecting the current play
+ * position within a song.
  */
 void
 change_pos( Panel_item item, int value, Event *event )
@@ -1049,6 +1288,8 @@ change_pos( Panel_item item, int value, Event *event )
 	char	time[6];
 	int	value_left;
 
+     if( event_is_up( event )) {
+	
 	time_wanted = value;
 
 	if (cur_cdmode == STOPPED && cur_track > 0)
@@ -1066,13 +1307,18 @@ change_pos( Panel_item item, int value, Event *event )
 
 		xv_set(Workman_window1->tracktimer, PANEL_LABEL_STRING, time,
 			NULL);
+/*
+ * Set the wanted track position.
+ */
 		cur_pos_rel = time_wanted;
 		cur_frame = cd->trk[cur_track - 1].start + time_wanted * 75;
 		cur_pos_abs = cur_frame / 75;
-	}
 
-	if (xv_get(Workman_goodies->abrepeat, PANEL_VALUE))
+	} /* STOPPED */
+
+	   if (xv_get(Workman_goodies->abrepeat, PANEL_VALUE))
 		xv_set(Workman_goodies->abrepeat, PANEL_VALUE, FALSE, NULL);
+     }	
 }
 
 /*
@@ -1181,20 +1427,48 @@ set_volume( Panel_item item, int value, Event *event )
 void
 figure_volume( window1_objects *ip )
 {
-	int volume = 0, old_manual = manual_volume;
+	int		volume = 0, old_manual = manual_volume;
+	int		balance;
+	int		max;
+	static int	oldvol = -1, oldbal = -1;
 
-	if (! manual_volume)
+	/*
+	 * See if some other process has fiddled with the volume level.
+	 */
+	balance = cur_balance;
+	volume = read_initial_volume(100);
+
+	if (cur_balance == oldbal)
+		cur_balance = balance;
+
+	if (oldvol >= 0 && volume != oldvol)
 	{
-		if (cur_track)
-			volume = get_default_volume(cur_track);
-		if (! volume)
-			volume = get_default_volume(0);
+		max = xv_get(ip->volume, PANEL_MAX_VALUE);
+		xv_set(ip->volume, PANEL_VALUE, (volume * max) / 100, NULL);
+		manual_volume = 1;
 	}
-	if (! volume)
-		volume = xv_get(ip->volume, PANEL_VALUE);
-	xv_set(ip->volume, PANEL_VALUE, volume, NULL);
-	set_volume(ip->volume, volume, NULL);
-	manual_volume = old_manual;
+	else
+	{
+		volume = 0;
+
+		if (! manual_volume)
+		{
+			if (cur_track)
+				volume = get_default_volume(cur_track);
+			if (! volume)
+				volume = get_default_volume(0);
+		}
+		if (! volume)
+			volume = xv_get(ip->volume, PANEL_VALUE);
+		xv_set(ip->volume, PANEL_VALUE, volume, NULL);
+		set_volume(ip->volume, volume, NULL);
+		manual_volume = old_manual;
+
+		read_initial_volume(100);
+		oldbal = cur_balance;
+	}
+
+	oldvol = volume;
 }
 
 /*
@@ -1303,7 +1577,7 @@ start_repeating( item, value, event)
 	Event		*event;
 {
 	cur_firsttrack = cur_lasttrack = -1;
-	play_chunk(mark_a, mark_b);
+	play_chunk(mark_a, mark_b, mark_a);
 } /* start_repeating() */
 
 /*
@@ -1803,7 +2077,7 @@ byebye( Notify_client c, Destroy_status s )
 	if (s == DESTROY_CHECKING && cur_cdmode != EJECTED)
 	{
 		keep_settings(Workman_window1);
-		if (info_modified)
+		if (info_modified && xv_get(Workman_popup1->button1, XV_SHOW))
 		{
 			xv_set(wannasave, XV_SHOW, TRUE, NULL);
 			if (confirmsave)
@@ -1958,3 +2232,46 @@ send_signal( char *cmd, char *pidfile )
 
 	exit(0);
 }
+
+/*
+ * Strip -geometry and -Ws arguments from the command line.  (Actually,
+ * retain the position part of the argument, if any, in the case of
+ * -geometry.)
+ */
+void strip_geometry( int *argc, char *argv[] )
+{
+	int	i, x;
+
+	for (i = 1; i < *argc; i++)
+	{
+		if (! strcmp(argv[i], "-geometry"))
+		{
+			i++;
+			if (i == *argc)
+				break;
+			for (x = 0; argv[i][x] != '\0'; x++)
+				if (argv[i][x] == '-' || argv[i][x] == '+')
+				{
+					argv[i] += x;
+					break;
+				}
+			
+			if (argv[i][x] == '\0')
+			{
+				/* No position argument -- strip this puppy. */
+				i--;
+				for (x = i + 2; x < *argc; x++)
+					argv[x - 2] = argv[x];
+
+				*argc = x - 2;
+			}
+		}
+
+		if (! strcmp(argv[i], "-Ws"))
+		{
+			for (x = i + 3; x < *argc; x++)
+				argv[x - 3] = argv[x];
+			*argc = x - 3;
+		}
+	}
+}
